unit QzPDFPrinter2;
{*********************************************************}
{*                    QzPDFPrinter.pas                   *}
{*                Quick Zip Html->Pdf Printer            *}
{*     Copyright (c) 2003 Joseph Leung Yat Chun          *}
{*                 All rights reserved.                  *}
{*********************************************************}

interface

uses
  SysUtils, Classes, Dialogs, Graphics, Jpeg, QzMiniHtml2, QzHtmlLabel2,
  /////////////////////////////////////////////////////////////////////
  //*              If you experiense error here....                 *//
  /////////////////////////////////////////////////////////////////////
  //* Please make sure you have PowerPDF installed in search path,  *//
  //*  or Remove this component from package!!                      *//
  //* http://www.est.hi-ho.ne.jp/takeshi_kanno/powerpdf/index.html  *//
  /////////////////////////////////////////////////////////////////////
  PdfDoc, PdfTypes, PdfFonts, PdfImages, PdfJpegImage, PReport;

type
  TPDFPrinter = class(TDummyPrinter)
                  private
                     PageStartY : integer;
                     FOutFile: TFileStream;
                     FTextUnderLine : boolean;
                     FTextStrikeOut : boolean;
                     FTextStyle : TTextStyle;
                     FFontColor: TColor;
                     FFontName: TPRFontName;
                     FFontSize: Single;
                     FFontBold: boolean;
                     FFontItalic: boolean;
                     function TranslateY(Y : Extended) : Extended;
                     procedure NextPage;
                  public
                     FDoc: TPdfDoc;
                     procedure Init(Height : integer); override;
                     procedure Save; override;
                     function GetWidth(Font : TFont; S : String) : integer; override;
                     procedure PicOut(X, Y: Single; FXObjectName : string; Graphic : TGraphic; Height, Width : integer); override;
                     procedure TextOut(X, Y: Single; S: string);  override;
                     procedure DrawLine(X1, Y1, X2, Y2, Width: Single); override;
                     procedure Rectangle(X1, Y1, X2, Y2, Width : Single); override;
                     procedure Ellipse(X1,Y1, Width, Height, LineWidth : Single); override;
                     procedure SetFont(Font : TFont; TextStyle : TTextStyle); override;
                     procedure SetPenColor(Color : TColor); override;
                     procedure SetBrushColor(Color : TColor); override;

                     constructor Create(AOwner : TComponent); override;
                     destructor Destroy; override;
                  end;

  TQzPDFPrinter2 = class(TComponent)
  private
    { Private declarations }
    MiniHtml : TQzMiniHtml2;
    Printer : TPDFPrinter;
  protected
    { Protected declarations }
  public
    { Public declarations }
    Filename : string;
    procedure Print;
    procedure UpdateMiniHtml;
  published
    { Published declarations }
    property QzMiniHtml2 : TQzMiniHtml2 read MiniHtml write MiniHtml;
  end;

procedure Register;

implementation

procedure TPDFPrinter.PicOut(X, Y: Single; FXObjectName : string; Graphic : TGraphic; Height, Width : integer);
begin
  with FDoc do
  begin
  if FDoc.GetXObject(FXObjectName) = nil then
    if (Graphic is TJpegImage) then
      AddXObject(FXObjectName, CreatePdfImage(Graphic, 'Pdf-Jpeg')) else
      AddXObject(FXObjectName, CreatePdfImage(Graphic, 'Pdf-Bitmap'));

  TranslateY(Y+Height);
  Canvas.DrawXObject(X, TranslateY(Y), Width, Height, FXObjectName);
  end;
end;

procedure TPDFPrinter.TextOut(X, Y: Single; S: string);
var k : string;
    YY,Modifier : extended;
begin
  with FDoc.Canvas do
  begin
    k := QzMinihtml2.Replace(S,#13#10,'');
    YY := TranslateY(Y);
    BeginText;
    Modifier := FFontsize / 5;
    if FTextStyle = isSuperScript then
      YY := YY + modifier;
    if FTextStyle = isSubScript then
      YY := YY - modifier;
    MoveTextPoint(X, YY);
    ShowText(k);
    EndText;
    if FTextUnderLine then
      begin
       YY := YY -2;//Dec(YY,2);
       SetDash([0], 0);;
       SetRGBStrokeColor(FFontColor);
       SetLineWidth(0.5);
       MoveTo(X, YY);
       LineTo(X+TextWidth(S), YY);
       Stroke;
       Newpath;
     end;
     if FTextStrikeout then
      begin
       YY := YY -2;
       SetDash([0], 0);;
       SetRGBStrokeColor(FFontColor);
       SetLineWidth(0.5);
       MoveTo(X, YY+(FFontSize / 2));
       LineTo(X+TextWidth(S), YY+(FFontSize / 2));
       Stroke;
       Newpath;
     end;

  end;
end;

procedure TPDFPrinter.DrawLine(X1, Y1, X2, Y2, Width: Single);
var YY1, YY2 : Single;
begin
  with FDoc.Canvas do
  begin
    SetLineWidth(Width);
    if Y2 > Y1 then
      begin
        YY1 := Y1;
        YY2 := Y2;
      end else
      begin
        YY2 := Y2;
        YY1 := Y1;
      end;

    MoveTo(X1, TranslateY(YY2));
    SetDash([0], 0);;
    SetLineWidth(Width);
    LineTo(X2, TranslateY(YY1));
    Closepath;
    Fill;
  end;
end;

procedure TPDFPrinter.Rectangle(X1, Y1, X2, Y2, Width: Single);
var YY1, YY2 : Single;
begin
  with FDoc.Canvas do
  begin
  SetLineWidth(Width);
    if Y2 > Y1 then
      begin
        YY1 := Y1;
        YY2 := Y2;
      end else
      begin
        YY2 := Y2;
        YY1 := Y1;
      end;
    SetLineWidth(Width);

    Moveto(x1,TranslateY(YY1));
    Lineto(x2,TranslateY(YY1));
    Lineto(x2,TranslateY(YY2));
    Lineto(x1,TranslateY(YY2));
    Lineto(x1,TranslateY(YY1));
    ClosepathFillStroke
  end;
end;

procedure TPDFPrinter.Ellipse(X1,Y1, Width, Height, LineWidth : Single);
begin
    with FDoc.Canvas do
    begin
    SetLineWidth(LineWidth);
    SetPenColor(clBlack);
    SetBrushColor(clBlack);
    Ellipse(X1,TranslateY(Y1),4,4);
    ClosepathFillStroke
    end;
end;

function GetFontClassName(FFontColor: TColor; FFontName: TPRFontName; FFontSize: Single;
                FFontBold: boolean; FFontItalic: boolean) : string;
begin
  if FFontBold then
    if FFontItalic then
      result := PDFFONT_CLASS_BOLDITALIC_NAMES[ord(FFontName)]
    else
      result := PDFFONT_CLASS_BOLD_NAMES[ord(FFontName)]
  else
    if FFontItalic then
      result := PDFFONT_CLASS_ITALIC_NAMES[ord(FFontName)]
    else
      result := PDFFONT_CLASS_NAMES[ord(FFontName)];
end;

procedure TPDFPrinter.SetFont(Font : TFont; TextStyle : TTextStyle);
var Modifier : extended;
begin

  if (Lowercase(Font.Name) = 'arial') or
     (Lowercase(Font.Name) = 'comic sans ms')
                   then FFontName := fnArial else
  if (Lowercase(Font.Name) = 'times new roman')
                   then FFontName := fnTimesRoman else
                        FFontName := fnFixedWidth;
  FFontBold   := (FsBold in Font.Style);
  FFontItalic := (FsItalic in Font.Style);
  FFontSize   := Font.Size+2;
  FFontColor  := Font.Color;
  FDoc.Canvas.SetRGBStrokeColor(FFontColor);
  FTextUnderLine := (FsUnderLine in Font.Style);
  FTextStrikeOut := (fsStrikeOut in Font.Style);
  FTextStyle := TextStyle;
  Modifier := FFontSize / 5;
  if TextStyle <> isNormal then
    FFontSize := FFontSize - Modifier;
  FDoc.Canvas.SetFont(GetFontClassName(FFOntColor,FFontName,FFontSize,FFontBold,FFontItalic),FFontSize);
  FDoc.Canvas.SetRGBFillColor(FFontColor);
end;

procedure TPDFPrinter.SetPenColor(Color : TColor);
begin
  FDoc.Canvas.SetRGBStrokeColor(Color);
end;

procedure TPDFPrinter.SetBrushColor(Color : TColor);
begin
  FDoc.Canvas.SetRGBFillColor(Color)
end;

procedure TPDFPrinter.NextPage;
begin
      FDoc.AddPage;
      FDoc.Canvas.PageWidth := PageWidth;
      FDoc.Canvas.PageHeight := PageHeight;
      Inc(PageStartY,ContentHeight);
      Inc(CurrentPage);
end;

function TPDFPrinter.TranslateY(Y : Extended) : Extended;
begin
  Result := Y - PageStartY;
  if Y - PageStartY < ContentHeight then
      Result := Y - PageStartY else
      while Y - PageStartY > ContentHeight do
      begin
      NextPage;
      Result := Y - PageStartY;
      end;
  Result := PageHeight - Result - HeaderHeight;
end;

procedure TPDFPrinter.Init(Height : integer);
begin
  if Height > PageHeight then PageHeight := Height + 50;
  FDoc.DefaultPageHeight := PageHeight;
  FDoc.DefaultPageWidth := PageWidth;
  ContentHeight := PageHeight - HeaderHeight - FooterHeight;
  Pages := (Height div ContentHeight);
  if (Height mod ContentHeight > 0) then Inc(Pages);
  FDoc.NewDoc;
  FDoc.AddPage;
  FDoc.Canvas.PageWidth := PageWidth;
  FDoc.Canvas.PageHeight := PageHeight;
  CurrentPage := 1;
  PageStartY := 0;
  FDoc.Canvas.SetCharSpace(0);

end;

procedure TPDFPrinter.Save;
begin
  FOutFile := TFileStream.Create(OutputFilename, fmCreate);
  try
    FDoc.SaveToStream(FOutFile);
  finally
    FOutfile.free;
    FDoc.FreeDoc;
  end;
end;

constructor TPDFPrinter.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  OutputFilename := 'Output.pdf';
  FDoc := TPdfDoc.Create;
  PageHeight := FDoc.DefaultPageHeight;
  PageWidth := FDoc.DefaultPageWidth;
  {$IFNDEF NOZLIB}
  FDoc.CompressionMethod := cmFlateDecode;
  {$ENDIF}
end;

destructor TPDFPrinter.Destroy;
begin
  FDoc.Free;
  inherited Destroy;
end;

function TPDFPrinter.GetWidth(Font : TFont; S : String) : integer;
begin
  SetFont(Font,isNormal);
  Result := Round(FDoc.Canvas.TextWidth(S));
end;

procedure TQzPDFPrinter2.UpdateMiniHtml;
var Tag1 : aTag;
    i, CurrentX, CurrentY, MaxHeight, LineNumber : integer;
procedure NewLine;
begin
  Inc(CurrentY,MaxHeight+Border);
  MaxHeight := 0;
  CurrentX := Border;
  Inc(LineNumber);
end;
begin
  CurrentX := Border;
  CurrentY := Border;
  MaxHeight := 0;
  LineNumber := 0;
  if Assigned(MiniHtml) then
  With MiniHtml do
    begin
      for i := 0 to TagList.Count -1 do
        begin
          Tag1 := TagList.items[i];
          If (Tag1 is aText) then
           with (Tag1 as aText) do
            begin
              Printer.SetFont(Font,isNormal);
              //Height := aText(Tag1).Font.Size+10;
              Width := Round(Printer.FDoc.Canvas.TextWidth(Text))+1;
              //if (CurrentY > Printer.PageWidth) then NewLine;
            end;
          while aTag(Tag1).LineNumber > LineNumber do
               NewLine;
          if  aTag(Tag1).Height > MaxHeight then  MaxHeight := aTag(Tag1).Height;
          aTag(Tag1).Left := CurrentX;
          aTag(Tag1).Top := CurrentY;
          Inc(CurrentX,aTag(Tag1).Width);
        end;
    end;

end;


procedure TQzPDFPrinter2.Print;
begin
  if Assigned(MiniHtml) then
  begin
  if Assigned(MiniHtml.HtmlPrinter) then
    MiniHtml.HtmlPrinter.Free;
  Printer := TPdfPrinter.Create(MiniHtml);
  if Filename <> '' then
    Printer.OutputFilename := Filename;

  if (MiniHtml.TagList.Count = 0) and (MiniHtml.Owner is TQzHtmlLabel2) then
    TQzHtmlLabel2(MiniHtml.Owner).Paint;

  MiniHtml.HtmlPrinter := Printer;
  MiniHtml.HtmlPrinter.Init(MiniHtml.CacheBitmap.Height);
  //UpdateMiniHtml;
  //MiniHtml.width := 100;
  MiniHtml.Update(Printer);
  MiniHtml.Print;
  MiniHtml.HtmlPrinter.Save;
  if Assigned(MiniHtml.HtmlPrinter) then
    MiniHtml.HtmlPrinter.Free;
  MiniHtml.HtmlPrinter := nil;
  MiniHtml.Update;

  end;
end;


procedure Register;
begin
  RegisterComponents('QZip', [TQzPDFPrinter2]);
end;

end.
